<?php

namespace Drupal\external_script_sri\Form;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Extension\ExtensionDiscovery;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Yaml;

/**
 * Configure JS Scanner settings for this site.
 */
class SriConfigurationForm extends ConfigFormBase {

  /**
   * The extension discovery service.
   *
   * @var \Drupal\Core\Extension\ExtensionDiscovery
   */
  protected $extensionDiscovery;

  /**
   * The Yaml service.
   *
   * @var \Symfony\Component\Yaml\Yaml
   */
  protected $yaml;

  /**
   * Constructs a new SriConfigurationForm instance.
   *
   * @param \Drupal\Core\Extension\ExtensionDiscovery $extension_discovery
   *   The extension discovery service.
   * @param \Symfony\Component\Yaml\Yaml $yaml
   *   The Yaml service.
   */
  public function __construct(ExtensionDiscovery $extension_discovery, Yaml $yaml) {
    $this->extensionDiscovery = $extension_discovery;
    $this->yaml = $yaml;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      new ExtensionDiscovery(\Drupal::root()),
      $container->get('external_script_sri.yaml')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'external_script_sri.sri_configuration.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'sri_configuration_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('external_script_sri.sri_configuration.settings');

    // Discover all module and theme paths.
    $modules_and_themes = array_merge(
      $this->extensionDiscovery->scan('module'),
      $this->extensionDiscovery->scan('theme')
    );

    $libraries = [];
    foreach ($modules_and_themes as $extension) {
      $libraries_yml = $extension->getPath() . '/' . $extension->getName() . '.libraries.yml';
      if (file_exists($libraries_yml)) {
        $parsed_libraries = $this->yaml::parseFile($libraries_yml);
        foreach ($parsed_libraries as $lib_name => $lib_info) {
          $libraries[$extension->getName()][$lib_name] = $lib_info;
        }
      }
    }

    // Manage the structure.
    $jsPaths = [];
    foreach ($libraries as $moduleName => $libraries) {
      foreach ($libraries as $libraryName => $libraryData) {
        if (isset($libraryData['js']) && is_array($libraryData['js'])) {
          foreach ($libraryData['js'] as $jsPath => $jsAttr) {
            $external_js_files = array_filter([$jsPath], function ($path) {
              return strpos($path, 'https://') === 0 || strpos($path, '//') === 0 || strpos($path, 'http://') === 0;
            });
            if (!empty($external_js_files)) {
              $jsPaths[$moduleName][$libraryName] = $external_js_files;
            }
            // $jsPaths[$moduleName][$libraryName][] = $jsPath;
          }
        }
      }
    }

    // Loop through the jsPaths array.
    $table = [];
    $i = 0;
    foreach ($jsPaths as $module => $libraries) {
      foreach ($libraries as $library => $jspaths) {
        foreach ($jspaths as $jspath) {
          $table[$i]['module'] = $module;
          $table[$i]['library'] = $library;
          $table[$i]['js'] = $jspath;
          $i++;
        }
      }
    }

    $js_library = $config->get('js_library');
    // Prepare saved configuration.
    $saved = [];
    if (!empty($js_library) && $js_library[0] !== NULL) {
      foreach ($js_library as $item) {
        $saved[$item['js_path']]['sri_hash'] = $item['sri_hash'];
        $saved[$item['js_path']]['crossorigin'] = $item['crossorigin'];
        $saved[$item['js_path']]['mark_as_sensitive'] = $item['mark_as_sensitive'];
      }
    }

    $form['js_library'] = [
      '#type' => 'table',
      '#caption' => $this->t('SRI Configuration: Add external JavaScript library paths with the ability to include SRI hashes and the `crossorigin` attribute. Use [this tool](https://www.srihash.org/) to generate SRI hashes based on the JavaScript URL.'),
      '#header' => [
        '#',
        $this->t('Module'),
        $this->t('Library'),
        $this->t('Mark as Sensitive'),
        $this->t('JS path'),
        $this->t('SRI Hash value'),
        $this->t('Crossorigin'),
      ],
      '#sticky' => TRUE,
    ];
    $i = 0;
    foreach ($table as $route => $item) {
      $form['js_library'][$i]['sl'] = [
        '#type' => 'markup',
        '#title' => $this->t('#'),
        '#title_display' => 'invisible',
        '#markup' => $i + 1,
      ];
      $form['js_library'][$i]['module'] = [
        '#type' => 'markup',
        '#title' => $this->t('Module'),
        '#title_display' => 'invisible',
        '#markup' => $item['module'],
      ];
      $form['js_library'][$i]['library'] = [
        '#type' => 'markup',
        '#title' => $this->t('Library Name'),
        '#title_display' => 'invisible',
        '#markup' => $item['library'],
      ];
      $form['js_library'][$i]['mark_as_sensitive'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Mark as Sensitive'),
        '#title_display' => 'invisible',
        '#default_value' => $saved[$item['js']]['mark_as_sensitive'] ?? FALSE,
      ];
      $form['js_library'][$i]['js_path'] = [
        '#type' => 'textarea',
        '#title' => $this->t('JS Path'),
        '#title_display' => 'invisible',
        '#default_value' => $item['js'],
        '#disabled' => TRUE,
      ];
      $form['js_library'][$i]['sri_hash'] = [
        '#type' => 'textarea',
        '#title' => $this->t('SRI Hash'),
        '#title_display' => 'invisible',
        '#default_value' => isset($saved[$item['js']]['sri_hash']) ? Xss::filter($saved[$item['js']]['sri_hash']) : '',
      ];
      $form['js_library'][$i]['crossorigin'] = [
        '#type' => 'select',
        '#title' => $this->t('Cross Origin'),
        '#title_display' => 'invisible',
        '#options' => [
          'anonymous' => $this->t('anonymous'),
          'use-credentials' => $this->t('use-credentials'),
        ],
        '#default_value' => $saved[$item['js']]['crossorigin'] ?? 'anonymous',
      ];
      $form['js_library'][$i]['module_hidden'] = [
        '#type' => 'hidden',
        '#value' => $item['module'],
      ];
      $form['js_library'][$i]['library_hidden'] = [
        '#type' => 'hidden',
        '#value' => $item['library'],
      ];
      $form['routetable'][$i]['route_alter_title']['route_hidden'] = [
        '#type' => 'hidden',
        '#value' => $route,
      ];
      $i++;
    }
    $form['js_library'][]['footer'] = [
      '#plain_text' => $this->t('Note: SRI (Subresource Integrity) may not work for some external paths due to dynamic content, CDN optimizations, caching mechanisms, or lack of support from third-party providers.'),
      '#wrapper_attributes' => [
        'colspan' => 7,
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $x = $form_state->getValue('js_library');
    $this->configFactory->getEditable('external_script_sri.sri_configuration.settings')
      ->set('js_library', $form_state->getValue('js_library'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
